<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with BWS REST API.
.DESCRIPTION
    Authenticate and call either "Create local user", "Delete user" or "Update local user" BWS REST API depending what actionType is used.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    TenantGuid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER actionType
    Action to perform. (AddUserToGroup or RemoveUserFromGroup)
.PARAMETER emailAddress
    Email address of the user to perform the action on.
.PARAMETER groupName
    Name of the group to add or remove user from.
.LINK
    https:// help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
.EXAMPLE
    .\AddRemoveUserToGroup.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -actionType AddUserToGroup -emailAddress pmorley@example.com -groupName testGroup
    .\AddRemoveUserToGroup.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType AD -username admin -password password -domain example -actionType RemoveUserFromGroup -emailAddress pmorley@example.com -groupName testGroup
#>

Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AddUserToGroup | RemoveUserFromGroup) actionType."
    )]
    [ValidateSet('AddUserToGroup', 'RemoveUserFromGroup')]
    [string]$actionType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="User email address to add or remove from group."
    )]
    [string]$emailAddress,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Name of group to add or remove user from."
    )]
    [string]$groupName
)

$global:authString = $null;
$global:uemBaseUrl = $null;

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $global:uemBaseUrl = "https://" + $uemHostAndPort + "/" + $tenantGuid + "/api/v1";

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    # Create body for REST call
    $body = @{}
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));
    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = $global:uemBaseUrl + "/util/authorization";

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        $global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:authString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
    * Get user by email address.
    *
    * @param email
    *       Email of the user to get.
    * @return
    *       User that was found.
#>
function getUser($email)
 {
    $methodName = "getUser()";
    enterMethod($methodName);
    $response = $null;

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:uemBaseUrl + "/users?query=emailAddress=" + $email;

    # Invoke RESTMethod to call users API to obtain users in the system
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
        if (($response.users -ne $null) -and ($response.users.length -eq 1))
        {
            Write-Host([string]::Format("User with email address '{0}' was found", $email));
        }
        else
        {
            Write-Host([string]::Format("User with email address '{0}' not found", $email));
            exit(1);
        }
    }
    catch
    {
         handleError $_;
    }
    exitMethod($methodName);
    return $response;
 }

<#
    * Get group by name.
    *
    * @param name
    *       Name of the group to get.
    * @return
    *       Group that was found.
#>
function getGroup($name)
{
    $methodName = "getGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.groups-v1+json");

    $request = $global:uemBaseUrl + "/groups?query=name=" + $name;

    # Invoke RESTMethod to call users API to obtain users in the system
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
        if (($response.groups -ne $null) -and ($response.groups.length -eq 1))
        {
            Write-Host([string]::Format("Group with name '{0}' was found", $name));
        }
        else
        {
            Write-Host([string]::Format("Group with name '{0}' not found.", $name));
            exit(1);
        }
    }
    catch
    {
         handleError $_;
    }

    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $response;
}

<#
    * Add user to group.
    *
    * @param userGuid
    *       Guid of the user.
    * @param name
    *       Guid of the group.
    * @return
    *       True if user is added, false otherwise.
#>
function addUserToGroup($userGuid, $groupGuid)
{
    $methodName = "addUserToGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));
    $returnValue = $false;
    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    # Create body for REST call
    $body = @{};
    $innerBody = @(@{
    "guid" = $userGUID;
    });
    $body.add("users",$innerBody);

    $request = $global:uemBaseUrl + "/groups/" + $groupGUID + "/users";

    # Invoke RESTMethod to call users API to obtain users in the system
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $true;
}


<#
    * Remove user from group.
    *
    * @param userGuid
    *       Guid of the user.
    * @param name
    *       Guid of the group.
    * @return
    *       True if user is removed, false otherwise.
#>
function removeUserFromGroup($userGuid, $groupGuid)
{
    $methodName = "removeUserFromGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    # Create body for REST call
    $body = @{};
    $innerBody = @(@{
    "guid" = $userGUID;
    });
    $body.add("users",$innerBody);

    $request = $global:uemBaseUrl + "/groups/" + $groupGUID + "/users";

    # Invoke RESTMethod to call users API to obtain users in the system
    try
    {
        $response = Invoke-RestMethod -Method DELETE -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $true;
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

function main
{
    $methodName = "Main()";
    Write-Host([string]::Format("Entering {0}...", $methodName));
    try
    {
        if (setup)
        {
            $userSearchResponse = getUser $emailAddress;
            $groupNameSearchResponse = getGroup $groupName;
            if($actionType -eq "AddUserToGroup")
            {
                if(addUserToGroup $userSearchResponse.users.guid $groupNameSearchResponse.groups.guid)
                {
                    Write-Host([string]::Format("User '{0}' has been added to the group '{1}'", $emailAddress, $groupName));
                }
            }
            elseif($actionType -eq "RemoveUserFromGroup")
            {
                if(removeUserFromGroup $userSearchResponse.users.guid $groupNameSearchResponse.groups.guid)
                {
                    Write-Host([string]::Format("User '{0}' has been removed from the group '{1}'", $emailAddress, $groupName));
                }
            }
        }
    }
    catch
    {
        handleError $_;
    }
    exitMethod($methodName);
    Write-Host("Complete!");
    exit(0);
}

# Call main function.
main;